
# q-expansion of newform 1350.4.c.b, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1350
weight = 4
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[1001, [1, 0]], [1027, [-1, 0]]]
aps_data = [[0, 2], [0, 0], [0, 0], [0, 29], [-57, 0], [0, -20], [0, -72], [106, 0], [0, -174], [210, 0], [47, 0], [0, 2], [-6, 0], [0, -218], [0, 474], [0, -81], [-84, 0], [56, 0], [0, -142], [360, 0], [0, 1159], [160, 0], [0, -735], [954, 0], [0, 191], [-363, 0], [0, 628], [0, 675], [-1730, 0], [0, -1866], [0, 1379], [579, 0], [0, 654], [3004, 0], [-1803, 0], [2459, 0], [0, -196], [0, 1564], [0, 1974], [0, 2217], [2475, 0], [1568, 0], [1140, 0], [0, -2045], [0, -3735], [-1163, 0], [2126, 0], [0, 2752], [0, -3972], [-4502, 0], [0, -4842], [5334, 0], [-3994, 0], [-1008, 0], [0, -924], [0, -1014], [2970, 0], [245, 0], [0, 4376], [240, 0], [0, 6838], [0, 5118], [0, -5560], [7662, 0], [0, -3485], [0, -7059], [9290, 0], [0, -3814], [0, 1929], [6586, 0], [0, 6042], [3762, 0], [0, -7261], [0, -1640], [7396, 0], [0, 4992], [9453, 0], [0, 8588], [-1716, 0], [9889, 0], [-5556, 0], [-2104, 0], [7614, 0], [0, -7805], [5209, 0], [0, -4236], [16002, 0], [0, 7319], [9483, 0], [0, -10793], [0, 2583], [1254, 0], [0, 17336], [-15171, 0], [-8930, 0], [0, -15210], [-19641, 0], [22428, 0], [0, 8152], [-2860, 0], [0, -9664], [0, 14859], [0, -8193], [16572, 0], [-6244, 0], [0, -14794], [0, -26769], [0, 3078], [-1002, 0], [-20653, 0], [0, 27128], [0, -24518], [0, -474], [1132, 0], [6725, 0], [-21126, 0], [0, -19460], [0, -11664], [0, 3345], [-9393, 0], [-1762, 0], [0, -25517], [0, 26898], [0, -23940], [23060, 0], [-14175, 0], [8692, 0], [29556, 0], [0, -36691], [0, 19798], [21976, 0], [0, 13236], [-6325, 0], [0, -3238], [-40416, 0], [4759, 0], [0, -27414], [0, 6176], [0, 6879], [16902, 0], [24086, 0], [7854, 0], [0, -5771], [0, -17568], [-31322, 0], [-41856, 0], [0, -15662], [0, -39864], [9160, 0], [0, -5076], [0, 14978], [-22860, 0], [0, 32506], [0, 35868], [0, -33586], [-28902, 0], [-28271, 0], [-19140, 0], [0, 31619], [-20913, 0], [0, 17529], [0, 53604], [0, 11117], [-27297, 0], [0, 25086], [0, 20982], [11477, 0], [0, 8588], [-21899, 0], [0, 22686], [56091, 0], [55262, 0], [39234, 0], [0, 58534], [50605, 0], [-39162, 0], [45470, 0], [-32409, 0], [0, -677], [24568, 0], [0, 33563], [16251, 0], [0, -57854], [0, -16812], [0, 28500], [-52791, 0], [0, -36826], [0, -46496], [69397, 0], [-48834, 0], [0, 23758], [0, -24765], [7454, 0], [37599, 0], [0, 59508], [0, -11100], [67007, 0], [0, -23654], [0, 36564], [0, 69156], [4569, 0], [-3373, 0], [0, -30760], [-8885, 0], [67707, 0], [0, -27738], [-44660, 0], [0, -61527], [-86010, 0], [61472, 0], [0, -29023], [44526, 0], [0, -7265], [0, 23001], [44808, 0], [46577, 0], [0, -18664], [-83838, 0], [0, 91998], [0, -30738], [-79522, 0], [-69356, 0], [-32376, 0], [0, 50380], [0, 28227], [73384, 0], [0, -106872], [-42030, 0], [0, -80512], [-11892, 0], [0, 78010], [21112, 0], [-1168, 0], [-6036, 0], [0, 51622], [0, -4908], [-20270, 0], [0, 22230], [67371, 0], [-13284, 0], [0, -51540], [-65230, 0], [0, -11117], [-43688, 0], [0, -480], [-14244, 0], [0, -27457], [35211, 0], [-45002, 0], [0, 32364], [0, -32380], [-57168, 0], [0, 34770], [-24329, 0], [0, 52851], [-34371, 0], [116804, 0], [0, 108056], [0, -43839], [0, 26057], [0, -58349], [0, 60657], [119698, 0], [0, 45736], [0, 103260], [51892, 0], [-93654, 0], [10950, 0], [0, -109964], [0, 118014], [-36376, 0], [0, 2864], [0, -22061], [-21161, 0], [0, 5393], [0, -37667], [0, -20409], [83104, 0], [120287, 0], [1788, 0], [0, -62364], [130295, 0], [0, -116346], [90074, 0], [0, 14438], [-35730, 0], [0, 77893], [0, 86259], [-44984, 0], [-78624, 0], [51378, 0], [0, -1449], [0, -67620], [89337, 0], [0, -126470], [-93450, 0], [71579, 0], [0, 114447], [-139203, 0], [0, -99034], [0, 57814], [0, -121149], [93283, 0]]
